==> mcss-0.0.43.tar.gz: mcss-0.0.43/src/mcss-properties/mcss-properties.cpp <==
// model properties simulator
// copyright 2008, 2009, 2010 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include <mcss/McssProperties.h>
#include <stdlib.h>
#include <string.h>

// calculate input concentration that produces specificed output concentration
int calculate_steady_state(McssPropertiesParameters &properties_parameters)
{
	unsigned int index;
	PropertiesResults properties_results;
		
	find_value_monotonic(properties_parameters, properties_results);
	
	if(properties_results.input == -1.0) {
		printf("-1.0 -1.0 %0.8f %d\n", properties_results.max_time, \
				properties_results.iterations);
		return 1;  // failed to find concentration
	}
	
	// save results model simulation as hdf5
	if(properties_parameters.save_hdf5) {
		// load mcss parameters
		SimulationParameters mcss_parameters;
		mcss_parameters.read(properties_parameters.mcss_parameters);
		
		// initialise model
		Psystem *psystem = new Psystem(mcss_parameters);
		
		// find input indices and compartments
		// compartment
		unsigned int comp_ind = psystem->getCompartmentIndexFromName( \
				properties_parameters.compartment);
		if(comp_ind == psystem->getNumCompartments()) {
			cerr << "error: compartment " << properties_parameters.compartment \
					<< " not found.\n";
			exit(EXIT_FAILURE);
		}
		// input species
		unsigned int input_ind = psystem->getSpeciesIndex( \
				properties_parameters.input_species);
		if(input_ind == psystem->getNumSpecies()) {
			cerr << "error: species " << properties_parameters.input_species \
					<< " not found.\n";
			exit(EXIT_FAILURE);
		}

		// set calculated level of input species
		psystem->setSpeciesConcentration(comp_ind, input_ind, \
				properties_results.input);
		
		// execute model
		psystem->execute(properties_results.max_time);
		
		// tidy up
		delete psystem;
	}
		
	// output results
	if(properties_parameters.output_header) {
		if(properties_results.num_output_species == 1)
			printf("input output max_time iterations\n");
		else {
			printf("input output_total");
			for(index = 0; index < properties_results.num_output_species; \
					index++)
				printf(" output_%d", index);
			printf(" max_time iterations\n");
		}
	}
	if(properties_results.num_output_species == 1)
		printf("%0.8E %0.8E %0.8f %d\n", properties_results.input, \
				properties_results.output, properties_results.max_time, \
				properties_results.iterations);
	else {
		printf("%0.8E %0.8E", properties_results.input, \
				properties_results.output);
		for(index = 0; index < properties_results.num_output_species; index++)
			printf(" %0.8E", properties_results.outputs[index]);
		printf(" %0.8f %d\n", properties_results.max_time, \
				properties_results.iterations);
	}
	
	// tidy up
	free(properties_results.outputs);
	
	return 0;
}

// calculate equilibrium dissociation constant using single site binding model
int calculate_Kd(McssPropertiesParameters &properties_parameters)
{
	PropertiesResults properties_results;
	double target;
	unsigned int comp_ind, output_ind;
	string::size_type index1, index2;
	int ret;

	// load mcss parameters
	SimulationParameters mcss_parameters;
	mcss_parameters.read(properties_parameters.mcss_parameters);
		
	// initialise model
	Psystem *psystem = new Psystem(mcss_parameters);
	
	// find index of output species compartment
	comp_ind = psystem->getCompartmentIndexFromName( \
			properties_parameters.compartment);
	if(comp_ind == psystem->getNumCompartments()) {
		cerr << "error: compartment " << properties_parameters.compartment << \
				" not found.\n";
		exit(EXIT_FAILURE);
	}
	
	// find concentration of output species
	// find output species indices
	target = 0.0;
	index1 = 0;
	do {
		index2 = properties_parameters.output_species.find(',', index1);
		if(index2 != string::npos)
			index2 -= index1;
		output_ind = psystem->getSpeciesIndex( \
				properties_parameters.output_species.substr(index1, index2));
		if(output_ind == psystem->getNumSpecies()) {
			cerr << "error: species " << properties_parameters. \
					output_species.substr(index1, index2) << " not found.\n";
			exit(EXIT_FAILURE);
		}
		target += psystem->getSpeciesConcentration(comp_ind, output_ind);
		if(index2 != string::npos)
			index1 = index2 + 1;
	} while(index2 != string::npos);
	
	// tidy up
	delete psystem;
	
	// set target concentration
	properties_parameters.target = target / 2.0;

	ret = calculate_steady_state(properties_parameters);
	
	return ret;
}

int main(int argc, char **argv)
{
	int ret;
	
	// parse command line and read in parameters
	if(argc == 1)
		#ifdef HAVE_MPI
		fprintf(stderr, "mcss-properties version %s (MPI)\n", VERSION);
		#else
		fprintf(stderr, "mcss-properties version %s\n", VERSION);
		#endif
	McssPropertiesParameters properties_parameters;
	properties_parameters.read(argc, argv);
	
	if(strcmp("steady_state_monotonic", properties_parameters.property) == 0) {
		ret = calculate_steady_state(properties_parameters);
		goto success;
	}
	
	if(strcmp("Kd_single_site", properties_parameters.property) == 0) {
		ret = calculate_Kd(properties_parameters);
		goto success;
	}
	
	// all done successfully
	success:
	
	return ret;
}

==> mcss-1.0.1.tar.gz: mcss-1.0.1/src/mcss-properties/mcss-properties.cpp <==
// model properties simulator
// copyright 2008, 2009, 2010 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include <mcss/McssProperties.h>
#include <stdlib.h>
#include <string.h>

// calculate input concentration that produces specificed output concentration
int calculate_steady_state(McssPropertiesParameters &properties_parameters)
{
	unsigned int index;
	PropertiesResults properties_results;
		
	find_value_monotonic(properties_parameters, properties_results);
	
	if(properties_results.input == -1.0) {
		printf("-1.0 -1.0 %0.8f %d\n", properties_results.max_time, \
				properties_results.iterations);
		return 1;  // failed to find concentration
	}
	
	// save results model simulation as hdf5
	if(properties_parameters.save_hdf5) {
		// load mcss parameters
		SimulationParameters mcss_parameters;
		mcss_parameters.read(properties_parameters.mcss_parameters);
		
		// initialise model
		Psystem *psystem = new Psystem(mcss_parameters);
		
		// find input indices and compartments
		// compartment
		unsigned int comp_ind = psystem->getCompartmentIndexFromName( \
				properties_parameters.compartment);
		if(comp_ind == psystem->getNumCompartments()) {
			cerr << "error: compartment " << properties_parameters.compartment \
					<< " not found.\n";
			exit(EXIT_FAILURE);
		}
		// input species
		unsigned int input_ind = psystem->getSpeciesIndex( \
				properties_parameters.input_species);
		if(input_ind == psystem->getNumSpecies()) {
			cerr << "error: species " << properties_parameters.input_species \
					<< " not found.\n";
			exit(EXIT_FAILURE);
		}

		// set calculated level of input species
		psystem->setSpeciesConcentration(comp_ind, input_ind, \
				properties_results.input);
		
		// execute model
		psystem->execute(properties_results.max_time);
		
		// tidy up
		delete psystem;
	}
		
	// output results
	if(properties_parameters.output_header) {
		if(properties_results.num_output_species == 1)
			printf("input_ss input output max_time iterations\n");
		else {
			printf("input_ss input output_total");
			for(index = 0; index < properties_results.num_output_species; \
					index++)
				printf(" output_%d", index);
			printf(" max_time iterations\n");
		}
	}
	if(properties_results.num_output_species == 1) {
		printf("%0.8E %0.8E %0.8E %0.8f %d\n", properties_results.input_ss, \
				properties_results.input, properties_results.output, \
				properties_results.max_time, properties_results.iterations);
		
	} else {
		printf("%0.8E %0.8E %0.8E", properties_results.input_ss, \
				properties_results.input, properties_results.output);
		for(index = 0; index < properties_results.num_output_species; index++)
			printf(" %0.8E", properties_results.outputs[index]);
		printf(" %0.8f %d\n", properties_results.max_time, \
				properties_results.iterations);
	}
	
	// tidy up
	free(properties_results.outputs);
	
	return 0;
}

// calculate equilibrium dissociation constant using single site binding model
int calculate_Kd(McssPropertiesParameters &properties_parameters)
{
	PropertiesResults properties_results;
	double target;
	unsigned int comp_ind, output_ind;
	string::size_type index1, index2;
	int ret;

	// load mcss parameters
	SimulationParameters mcss_parameters;
	mcss_parameters.read(properties_parameters.mcss_parameters);
		
	// initialise model
	Psystem *psystem = new Psystem(mcss_parameters);
	
	// find index of output species compartment
	comp_ind = psystem->getCompartmentIndexFromName( \
			properties_parameters.compartment);
	if(comp_ind == psystem->getNumCompartments()) {
		cerr << "error: compartment " << properties_parameters.compartment << \
				" not found.\n";
		exit(EXIT_FAILURE);
	}
	
	// find concentration of output species
	// find output species indices
	target = 0.0;
	index1 = 0;
	do {
		index2 = properties_parameters.output_species.find(',', index1);
		if(index2 != string::npos)
			index2 -= index1;
		output_ind = psystem->getSpeciesIndex( \
				properties_parameters.output_species.substr(index1, index2));
		if(output_ind == psystem->getNumSpecies()) {
			cerr << "error: species " << properties_parameters. \
					output_species.substr(index1, index2) << " not found.\n";
			exit(EXIT_FAILURE);
		}
		target += psystem->getSpeciesConcentration(comp_ind, output_ind);
		if(index2 != string::npos)
			index1 = index2 + 1;
	} while(index2 != string::npos);
	
	// tidy up
	delete psystem;
	
	// set target concentration
	properties_parameters.target = target / 2.0;

	ret = calculate_steady_state(properties_parameters);
	
	return ret;
}

int main(int argc, char **argv)
{
	int ret;
	
	// parse command line and read in parameters
	if(argc == 1)
		#ifdef HAVE_MPI
		fprintf(stderr, "mcss-properties version %s (MPI)\n", VERSION);
		#else
		fprintf(stderr, "mcss-properties version %s\n", VERSION);
		#endif
	McssPropertiesParameters properties_parameters;
	properties_parameters.read(argc, argv);
	
	if(strcmp("steady_state_monotonic", properties_parameters.property) == 0) {
		ret = calculate_steady_state(properties_parameters);
		goto success;
	}
	
	if(strcmp("Kd_single_site", properties_parameters.property) == 0) {
		ret = calculate_Kd(properties_parameters);
		goto success;
	}
	
	// all done successfully
	success:
	
	return ret;
}
